# !/usr/bin/env python
import pika


class Microclient():
    def __init__(self, uri, exchange, qin=None, qout=None, process=lambda x: x, routing=None):
        self.uri = uri
        self.exchange = exchange
        self.qin = qin
        self.qout = qout
        self.process = process
        self.routing = routing
        self.connection = None
        self.channel = None
        if self.exchange == '':
            self.declare = self.bind

    def connection_open(self, connection):
        self.connection = connection
        self.connection.channel(self.channel_open)

    def channel_open(self, channel):
        self.channel = channel
        channel.queue_declare(self.declare, queue=self.qin)

    def declare(self, x):
        self.channel.queue_bind(self.bind, exchange=self.exchange, queue=self.qin)

    def bind(self, x):
        self.channel.basic_consume(self.eat_carrot, self.qin)

    def eat_carrot(self, channel, method, properties, body):
        pass

    def push_carrot(self, queue, payload):
        self.channel.basic_publish(self.exchange,
                                   queue,
                                   payload,
                                   pika.BasicProperties(content_type='text/json',
                                                        delivery_mode=1))

    def start(self):
        parameters = pika.URLParameters(self.uri)
        self.connection = pika.SelectConnection(parameters=parameters,
                                                on_open_callback=self.connection_open)

    def start_loop(self):
        try:
            self.connection.ioloop.start()
        except KeyboardInterrupt:
            self.connection.close()
            self.connection.ioloop.start()


from carrots import Microclient
from carrots.common.functions import apply

class Microconsumer(Microclient):
    def __init__(self, uri, exchange, qin, process, routing=None):
        super().__init__(uri, exchange, qin=qin, process=process, routing=routing)

    def declare(self, x):
        self.channel.queue_bind(self.bind, exchange=self.exchange, queue=self.qin, routing_key=self.routing)

    def eat_carrot(self, *args):
        apply(self.process, *args)

    def receive(self):
        self.start()
        self.start_loop()



from carrots import Microclient
import json

class Microproducer(Microclient):
    def __init__(self, uri, exchange, qout):
        super().__init__(uri, exchange, qout=qout)
        self.data = None

    def channel_open(self, channel):
        self.channel = channel
        self.channel.queue_declare(self.declare, queue=self.qout)

    def declare(self, x):
        self.channel.queue_bind(self.bind, exchange=self.exchange, queue=self.qout)

    def bind(self, *args):
        if isinstance(self.data,list):
            for item in self.data:
                #body = json.dumps(item)
                self.push_carrot(self.qout, item)
        else:
            self.push_carrot(self.qout, self.data)
        self.connection.close()
        self.connection.ioloop.start()

    def send(self, data):
        self.data = data
        self.start()
        self.start_loop()


from setuptools import setup
setup(name='carrots',
      version='0.1.12',
      description='Library to build RabbitMQ architectures like you were playing with lego blocks',
      url='http://github.com/croccam/carrots',
      author='Croccam',
      author_email='antcarri@gmail.com',
      license='MIT',
      install_requires=['pika'],
      packages=['carrots',
                'carrots.common'],
      zip_safe=False)